@@ -15,6 +15,7 @@ setMethod("[", signature(x="bed.matrix",i="numeric",j="missing", drop="missing")
       x@bed <- .Call('gg_extract_inds_indices', x@bed, i) 
       x@ped <- x@ped[i,]
       x@snps <- void_snps_stats(x@snps)
+      if(getOption("gaston.auto.set.stats", TRUE)) x <- set.stats(x, set.ped_stats = FALSE, verbose = FALSE)
       x
     } );
 
@@ -24,6 +25,7 @@ setMethod("[", signature(x="bed.matrix",i="logical",j="missing", drop="missing")
       x@bed <- .Call('gg_extract_inds_bool', x@bed, i) 
       x@ped <- x@ped[i,]
       x@snps <- void_snps_stats(x@snps)
+      if(getOption("gaston.auto.set.stats", TRUE)) x <- set.stats(x, set.ped_stats = FALSE, verbose = FALSE)
       x
     } );
   
@@ -36,6 +38,7 @@ setMethod("[", signature(x="bed.matrix",i="missing",j="numeric", drop="missing")
       x@mu <- x@mu[j]
       x@sigma <- x@sigma[j]
       x@ped <- void_ped_stats(x@ped)
+      if(getOption("gaston.auto.set.stats", TRUE)) x <- set.stats(x, set.snps_stats = FALSE, set.p = FALSE, set.mu_sigma = FALSE, verbose = FALSE)
       x
     } );
   
@@ -48,6 +51,7 @@ setMethod("[", signature(x="bed.matrix",i="missing",j="logical", drop="missing")
       x@mu <- x@mu[j]
       x@sigma <- x@sigma[j]
       x@ped <- void_ped_stats(x@ped)
+      if(getOption("gaston.auto.set.stats", TRUE)) x <- set.stats(x, set.snps_stats = FALSE, set.p = FALSE, set.mu_sigma = FALSE, verbose = FALSE)
       x
     } );
   
@@ -65,6 +69,7 @@ setMethod("[", signature(x="bed.matrix",i="logical",j="logical", drop="missing")
       x@ped <- x@ped[i,]
       x@ped <- void_ped_stats(x@ped)
       x@snps <- void_snps_stats(x@snps)
+      if(getOption("gaston.auto.set.stats", TRUE)) x <- set.stats(x, verbose = FALSE)
       x
     } );
   
@@ -83,6 +88,7 @@ setMethod("[", signature(x="bed.matrix",i="logical",j="numeric", drop="missing")
       x@ped <- x@ped[i,]
       x@ped <- void_ped_stats(x@ped)
       x@snps <- void_snps_stats(x@snps)
+      if(getOption("gaston.auto.set.stats", TRUE)) x <- set.stats(x, verbose = FALSE)
       x
     } );
 
@@ -100,6 +106,7 @@ setMethod("[", signature(x="bed.matrix",i="numeric",j="logical", drop="missing")
       x@ped <- x@ped[i,]
       x@ped <- void_ped_stats(x@ped)
       x@snps <- void_snps_stats(x@snps)
+      if(getOption("gaston.auto.set.stats", TRUE)) x <- set.stats(x, verbose = FALSE)
       x
     } );
   
@@ -118,6 +125,7 @@ setMethod("[", signature(x="bed.matrix",i="numeric",j="numeric", drop="missing")
       x@ped <- x@ped[i,]
       x@ped <- void_ped_stats(x@ped)
       x@snps <- void_snps_stats(x@snps)
+      if(getOption("gaston.auto.set.stats", TRUE)) x <- set.stats(x, verbose = FALSE)
       x
     } );
 
